@@ -128,6 +128,65 @@ static int swap_volnames(libubi_t libubi,
 	return ubi_rnvols(libubi, masternode, &rnvol);
 }
 
+/**
+ * check_ubi_alwaysremove - check the property always-remove for this image
+ * @img: image information
+ *
+ * Return: 1 if the property always-remove is true, otherwise 0.
+ */
+static int check_ubi_alwaysremove(struct img_type *img)
+{
+	char *always;
+	int ret = 0;
+
+	always = dict_get_value(&img->properties, "always-remove");
+
+	if (always && !strcmp(always, "true"))
+		ret = 1;
+
+	return ret;
+}
+
+static long long get_output_size(struct img_type *img)
+{
+	char *output_size_str = NULL;
+	long long bytes = img->size;
+
+	if (img->compressed) {
+		output_size_str = dict_get_value(&img->properties, "decompressed-size");
+
+		bytes = ustrtoull(output_size_str, 0);
+		if (errno) {
+			ERROR("decompressed-size argument: ustrtoull failed");
+			return -1;
+		}
+
+		if (bytes == 0) {
+			ERROR("UBIFS to be decompressed, but decompressed-size not valid");
+			return -1;
+		}
+		TRACE("Image is compressed, decompressed size %lld bytes", bytes);
+
+	} else if (img->is_encrypted) {
+
+		output_size_str = dict_get_value(&img->properties, "decrypted-size");
+
+		bytes = ustrtoull(output_size_str, 0);
+		if (errno){
+			ERROR("decrypted-size argument: ustrtoull failed");
+			return -1;
+		}
+
+		if (bytes < AES_BLK_SIZE) {
+			ERROR("Encrypted image size (%lld) too small", bytes);
+			return -1;
+		}
+		TRACE("Image is crypted, decrypted size %lld bytes", bytes);
+	}
+
+	return bytes;
+}
+
 static int update_volume(libubi_t libubi, struct img_type *img,
 	struct ubi_vol_info *vol)
 {
@@ -136,30 +195,11 @@ static int update_volume(libubi_t libubi, struct img_type *img,
 	char node[64];
 	int err;
 	char sbuf[128];
-	char *decrypted_size_str = NULL;
 	struct ubi_vol_info *repl_vol;
 
-	bytes = img->size;
-	if (img->is_encrypted) {
-
-		decrypted_size_str = dict_get_value(&img->properties, "decrypted-size");
-
-		bytes = ustrtoull(decrypted_size_str, 0);
-		if (errno){
-			ERROR("decrypted-size argument: ustrtoull failed");
-			return -1;
-		}
-
-		if (img->compressed) {
-			ERROR("Decryption of compressed UBI images not supported");
-			return -1;
-		}
-		if (bytes < AES_BLOCK_SIZE) {
-			ERROR("Encrypted image size (%lld) too small", bytes);
-			return -1;
-		}
-		TRACE("Image is crypted, decrypted size %lld bytes", bytes);
-	}
+	bytes = get_output_size(img);
+	if (bytes <= 0)
+		return -1;
 
 	if (!libubi) {
 		ERROR("Request to write into UBI, but no UBI on system");
@@ -289,7 +329,8 @@ static int resize_volume(struct img_type *cfg, long long size)
 		allocated_lebs = ubivol->vol_info.rsvd_bytes / mtd_info->dev_info.leb_size;
 
 		if (requested_lebs == allocated_lebs &&
-		    req_vol_type == ubivol->vol_info.type) {
+		    req_vol_type == ubivol->vol_info.type &&
+		    !check_ubi_alwaysremove(cfg)) {
 			TRACE("skipping volume %s (same size and type)",
 			      ubivol->vol_info.name);
 			return 0;
@@ -410,29 +451,9 @@ static int install_ubivol_image(struct img_type *img,
 	int ret;
 
 	if (check_ubi_autoresize(img)) {
-		long long bytes = img->size;
-
-		if (img->is_encrypted) {
-			char *decrypted_size_str = NULL;
-
-			decrypted_size_str = dict_get_value(&img->properties,
-							    "decrypted-size");
-
-			bytes = ustrtoull(decrypted_size_str, 0);
-			if (errno) {
-				ERROR("decrypted-size argument: ustrtoull failed");
-				return -1;
-			}
-
-			if (img->compressed) {
-				ERROR("Decryption of compressed UBI images not supported");
-				return -1;
-			}
-			if (bytes < AES_BLOCK_SIZE) {
-				ERROR("Encrypted image size (%lld) too small", bytes);
-				return -1;
-			}
-		}
+		long long bytes = get_output_size(img);
+		if (bytes <= 0)
+			return -1;
 
 		ret = resize_volume(img, bytes);
 		if (ret < 0) {
