@@ -11,6 +11,7 @@
 
 #include <stdlib.h>
 #include <stdbool.h>
+#include <string.h>
 #include "swupdate_status.h"
 
 #ifdef __cplusplus
@@ -32,8 +33,9 @@ typedef enum {
 	GET_STATUS,
 	POST_UPDATE,
 	SWUPDATE_SUBPROCESS,
-	REQ_INSTALL_DRYRUN,
-	SET_AES_KEY
+	SET_AES_KEY,
+	SET_UPDATE_STATE,	/* set bootloader ustate */
+	REQ_INSTALL_EXT
 } msgtype;
 
 /*
@@ -46,6 +48,27 @@ enum {
 	CMD_ENABLE	/* Enable or disable suricatta mode */
 };
 
+enum run_type {
+	RUN_DEFAULT,
+	RUN_DRYRUN,
+	RUN_INSTALL
+};
+
+#define SWUPDATE_API_VERSION 	0x1
+/*
+ * Install structure to be filled before calling
+ * ipc and async functions
+ */
+struct swupdate_request {
+	unsigned int apiversion;
+	sourcetype source;
+	enum run_type dry_run;
+	size_t len;
+	char info[512];
+	char software_set[256];
+	char running_mode[256];
+};
+
 typedef union {
 	char msg[128];
 	struct { 
@@ -54,6 +77,14 @@ typedef union {
 		int error;
 		char desc[2048];
 	} status;
+	struct {
+		struct swupdate_request req;
+		unsigned int len;    /* Len of data valid in buf */
+		char	buf[2048];   /*
+				      * Buffer that each source can fill
+				      * with additional information
+				      */
+	} instmsg;
 	struct {
 		sourcetype source; /* Who triggered the update */
 		int	cmd;	   /* Optional encoded command */
@@ -63,7 +94,7 @@ typedef union {
 				      * Buffer that each source can fill
 				      * with additional information
 				      */
-	} instmsg;
+	} procmsg;
 	struct {
 		char key_ascii[65]; /* Key size in ASCII (256 bit, 32 bytes bin) + termination */
 		char ivt_ascii[33]; /* Key size in ASCII (16 bytes bin) + termination */
@@ -78,7 +109,7 @@ typedef struct {
 
 char *get_ctrl_socket(void);
 int ipc_inst_start(void);
-int ipc_inst_start_ext(sourcetype source, size_t len, const char *info, bool dryrun);
+int ipc_inst_start_ext(void *priv, ssize_t size);
 int ipc_send_data(int connfd, char *buf, int size);
 void ipc_end(int connfd);
 int ipc_get_status(ipc_message *msg);
@@ -90,9 +121,12 @@ typedef int (*writedata)(char **buf, int *size);
 typedef int (*getstatus)(ipc_message *msg);
 typedef int (*terminated)(RECOVERY_STATUS status);
 int ipc_wait_for_complete(getstatus callback);
+void swupdate_prepare_req(struct swupdate_request *req);
 int swupdate_image_write(char *buf, int size);
 int swupdate_async_start(writedata wr_func, getstatus status_func,
-				terminated end_func, bool dryrun);
+				terminated end_func,
+				void *priv, ssize_t size);
+int swupdate_set_aes(char *key, char *ivt);
 
 #ifdef __cplusplus
 }   // extern "C"
