@@ -13,6 +13,7 @@
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
+#include <limits.h>
 
 #include "lkc.h"
 
@@ -770,10 +771,14 @@ int conf_write(const char *name)
 	} else
 		basename = conf_get_configname();
 
-	sprintf(newname, "%s%s", dirname, basename);
+	if (snprintf(newname, PATH_MAX+1, "%s%s", dirname, basename) < 0) {
+		return 1;
+	}
 	env = getenv("KCONFIG_OVERWRITECONFIG");
 	if (!env || !*env) {
-		sprintf(tmpname, "%s.tmpconfig.%d", dirname, (int)getpid());
+		if (snprintf(tmpname, PATH_MAX+1, "%s.tmpconfig.%d", dirname, (int)getpid()) < 0) {
+			return 1;
+		}
 		out = fopen(tmpname, "w");
 	} else {
 		*tmpname = 0;
